"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 07.12.20

"""
from dal_select2.views import Select2QuerySetView

from codelist.models import KonovaCode
from compensation.models import EcoAccount
from intervention.models import Intervention
from organisation.models import Organisation


class OrganisationAutocomplete(Select2QuerySetView):
    def get_queryset(self):
        if self.request.user.is_anonymous:
            return Organisation.objects.none()
        qs = Organisation.objects.all()
        if self.q:
            qs = qs.filter(name__icontains=self.q)
            qs = qs.order_by(
                "name"
            )
        return qs


class NonOfficialOrganisationAutocomplete(Select2QuerySetView):
    def get_queryset(self):
        if self.request.user.is_anonymous:
            return Organisation.objects.none()
        qs = Organisation.objects.all()
        if self.q:
            qs = qs.filter(
                name__icontains=self.q,
            )
            qs = qs.order_by(
                "name"
            )
        return qs


class EcoAccountAutocomplete(Select2QuerySetView):
    def get_queryset(self):
        if self.request.user.is_anonymous:
            return EcoAccount.objects.none()
        qs = EcoAccount.objects.filter(
            deleted=None,
            recorded__isnull=False,
        )
        if self.q:
            qs = qs.filter(
                identifier__icontains=self.q
            )
            qs = qs.order_by(
                "identifier"
            )
        return qs


class InterventionAutocomplete(Select2QuerySetView):
    def get_queryset(self):
        if self.request.user.is_anonymous:
            return Intervention.objects.none()
        qs = Intervention.objects.filter(
            deleted=None,
            users__in=[self.request.user],
        )
        if self.q:
            qs = qs.filter(
                identifier__icontains=self.q
            )
            qs = qs.order_by(
                "identifier"
            )
        return qs


class KonovaCodeAutocomplete(Select2QuerySetView):
    """
    Provides simple autocomplete functionality for codes

    Parameter support:
    * q: Search for a word inside long_name of a code
    * c: Search inside a special codelist

    """
    def dispatch(self, request, *args, **kwargs):
        # Retrieve 'c' for 'code' from request
        self.c = request.GET.get("c", "")
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        if self.request.user.is_anonymous:
            return KonovaCode.objects.none()
        qs = KonovaCode.objects.filter(
            is_active=True,
            is_leaf=True,
        )
        if self.c:
            qs = qs.filter(
                code_lists__in=[self.c]
            )
        if self.q:
            qs = qs.filter(
                long_name__icontains=self.q
            )
            qs.order_by(
                "long_name"
            )
        return qs